@@ -9,34 +9,34 @@ Sepia - Simple Emacs-Perl Interface
 From inside Emacs:
 
    M-x load-library RET sepia RET
-   M-x sepia-init RET
+   M-x sepia-repl RET
 
-At the prompt in the C<*perl-interaction*> buffer:
+At the prompt in the C<*sepia-repl*> buffer:
 
    main @> ,help
 
+For more information, please see F<sepia/index.html>.
+
 =cut
 
-$VERSION = '0.75';
+$VERSION = '0.90';
 @ISA = qw(Exporter);
 
 require Exporter;
 use strict;
+use B;
+use Sepia::Debug;               # THIS TURNS ON DEBUGGING INFORMATION!
 use Cwd 'abs_path';
 use Scalar::Util 'looks_like_number';
 use Module::Info;
 use Text::Abbrev;
-use Carp;
-use B;
 
-use vars qw($PS1 $dies $STOPDIE $STOPWARN %REPL %RK %REPL_DOC
-            $PACKAGE $WANTARRAY $PRINTER $STRICT $PRINT_PRETTY);
+use vars qw($PS1 %REPL %RK %REPL_DOC
+            $REPL_LEVEL $REPL_IN $REPL_OUT
+            $PACKAGE $WANTARRAY $PRINTER $STRICT $PRINT_PRETTY
+            $ISEVAL);
 
 BEGIN {
-    eval { require PadWalker; import PadWalker qw(peek_my) };
-    if ($@) {
-        *peek_my = sub { +{ } };
-    }
     eval { require Lexical::Persistence; import Lexical::Persistence };
     if ($@) {
         *repl_strict = sub {
@@ -54,6 +54,12 @@ BEGIN {
             0;
         };
     }
+    eval { require Module::CoreList };
+    if ($@) {
+        *Sepia::core_version = sub { '???' };
+    } else {
+        *Sepia::core_version = sub { Module::CoreList->first_release(@_) };
+    }
 }
 
 =head1 DESCRIPTION
@@ -61,7 +67,9 @@ BEGIN {
 Sepia is a set of features to make Emacs a better tool for Perl
 development.  This package contains the Perl side of the
 implementation, including all user-serviceable parts (for the
-cross-referencing facility see L<Sepia::Xref>).
+cross-referencing facility see L<Sepia::Xref>).  This document is
+aimed as Sepia developers; for user documentation, see
+L<sepia/index.html>.
 
 Though not intended to be used independent of the Emacs interface, the
 Sepia module's functionality can be used through a rough procedural
@@ -69,10 +77,21 @@ interface.
 
 =head2 C<@compls = completions($string [, $type])>
 
-Find a list of completions for C<$string> with glob type $type.
+Find a list of completions for C<$string> with glob type C<$type>,
+which may be "SCALAR", "HASH", "ARRAY", "CODE", "IO", or the special
+value "VARIABLE", which means either scalar, hash, or array.
 Completion operates on word subparts separated by [:_], so
 e.g. "S:m_w" completes to "Sepia::my_walksymtable".
 
+=head2 C<@compls = method_completions($expr, $string [,$eval])>
+
+Complete among methods on the object returned by C<$expr>.  The
+C<$eval> argument, if present, is a function used to do the
+evaluation; the default is C<eval>, but for example the Sepia REPL
+uses C<Sepia::repl_eval>.  B<Warning>: Since it has to evaluate
+C<$expr>, method completion can be extremely problematic.  Use with
+care.
+
 =cut
 
 sub _apropos_re($)
@@ -131,7 +150,13 @@ sub completions
         } _completions $str;
     } else {
         @ret = grep {
-            $type eq 'SCALAR' ? defined ${$_} : defined *{$_}{$type}
+            if ($type eq 'SCALAR') {
+                defined ${$_};
+            } elsif ($type eq 'VARIABLE') {
+                defined ${$_} || defined *{$_}{HASH} || defined *{$_}{ARRAY};
+            } else {
+                defined *{$_}{$type}
+            }
         } _completions $str;
         if (defined $infunc && defined *{$infunc}{CODE}) {
             my ($apre) = _apropos_re($str);
@@ -166,13 +191,36 @@ sub completions
             } lexicals($infunc);
         }
     }
+    ## Complete packages so e.g. "new B:T" -> "new Blah::Thing"
+    ## instead of "new Blah::Thing::"
+    if (!$type) {
+        @ret = map { /(.*)::$/ ? ($1, $_) : $_ } @ret;
+    }
     ## XXX: Control characters, $", and $1, etc. confuse Emacs, so
     ## remove them.
     grep {
-        !looks_like_number $_ && !/^[^\w\d_]$/ && !/^_</ && !/^[[:cntrl:]]/
+        length > 0 && !looks_like_number $_ && !/^[^\w\d_]$/ && !/^_</ && !/^[[:cntrl:]]/
     } map { s/^:://; $_ } @ret;
 }
 
+sub method_completions
+{
+    my ($x, $fn, $eval) = @_;
+    $x =~ s/^\s+//;
+    $x =~ s/\s+$//;
+    $eval ||= 'CORE::eval';
+    no strict;
+    return unless ($x =~ /^\$/ && ($x = $eval->("ref($x)")))
+        || $eval->('defined(%{'.$x.'::})');
+    unless ($@) {
+        my $re = _apropos_re $fn;
+        ## Filter out overload methods "(..."
+        return sort { $a cmp $b } map { s/.*:://; $_ }
+            grep { defined *{$_}{CODE} && /::$re/ && !/\(/ }
+                methods($x, 1);
+    }
+}
+
 =head2 C<@locs = location(@names)>
 
 Return a list of [file, line, name] triples, one for each function
@@ -310,7 +358,7 @@ sub mod_decls
 	my $sn = $_;
 	my $proto = prototype(\&{"$pack\::$sn"});
 	$proto = defined($proto) ? "($proto)" : '';
-	"sub $sn $proto;\n";
+	"sub $sn $proto;";
     } mod_subs($pack);
     return wantarray ? @ret : join '', @ret;
 }
@@ -450,6 +498,9 @@ sub tolisp($)
         } elsif (looks_like_number $thing) {
             ''.(0+$thing);
         } else {
+            ## XXX Elisp and perl have slightly different
+            ## escaping conventions, so we do this crap instead.
+            $thing =~ s/["\\]/\\$1/g;
             qq{"$thing"};
         }
     } elsif ($t eq 'GLOB') {
@@ -469,9 +520,9 @@ sub tolisp($)
     }
 }
 
-=head2 C<printer(\@res [, $iseval])>
+=head2 C<printer(\@res, $wantarray)>
 
-Print C<@res> appropriately on the current filehandle.  If C<$iseval>
+Print C<@res> appropriately on the current filehandle.  If C<$ISEVAL>
 is true, use terse format.  Otherwise, use human-readable format,
 which can use either L<Data::Dumper>, L<YAML>, or L<Data::Dump>.
 
@@ -479,15 +530,28 @@ which can use either L<Data::Dumper>, L<YAML>, or L<Data::Dump>.
 
 sub print_dumper
 {
+    eval { require Data::Dumper };
     local $Data::Dumper::Deparse = 1;
     local $Data::Dumper::Indent = 0;
+    local $_;
     no strict;
+    my $thing = @res > 1 ? \@res : $res[0];
     eval {
-        local $_ = Data::Dumper::Dumper(@res > 1 ? \@res : $res[0]);
+        $_ = Data::Dumper::Dumper($thing);
         s/^\$VAR1 = //;
         s/;$//;
-        $_;
     };
+    if (length $_ > ($ENV{COLUMNS} || 80)) {
+        $Data::Dumper::Indent = 2;
+        eval {
+            $_ = Data::Dumper::Dumper($thing);
+            s/\A\$VAR1 = //;
+            s/;\Z//;
+        };
+        s/\A\$VAR1 = //;
+        s/;\Z//;
+    }
+    $_;
 }
 
 sub print_plain
@@ -514,7 +578,7 @@ sub print_dump
     if ($@) {
         print_dumper;
     } else {
-        Data::Dump::dump;
+        Data::Dump::dump(\@res);
     }
 }
 
@@ -522,23 +586,24 @@ sub printer
 {
     no strict;
     local *res = shift;
-    my ($iseval, $wantarray) = @_;
+    my ($wantarray) = @_;
     @::__ = @res;
     $::__ = @res == 1 ? $res[0] : [@res];
     my $str;
-    if ($iseval) {
+    if ($ISEVAL) {
         $res = "@res";
-    } elsif (@res == 1 && (ref $res[0]) =~ /^PDL/) {
+    } elsif (@res == 1 && UNIVERSAL::can($res[0], '()')) {
+        # overloaded?
         $res = $res[0];
-    } elsif (!$iseval && $PRINT_PRETTY && @res > 1 && grep !ref $_, @res) {
-        $res = columnate(@res);
+    } elsif (!$ISEVAL && $PRINT_PRETTY && @res > 1 && !grep ref, @res) {
+        $res = columnate(sort @res);
         print $res;
         return;
     } else {
         $res = $PRINTER->();
     }
-    if ($iseval) {
-        print ';;;', length $res, "\n$::__\n";
+    if ($ISEVAL) {
+        print ';;;', length $res, "\n$res\n";
     } else {
         print "=> $res\n";
     }
@@ -567,15 +632,11 @@ Behavior is controlled in part through the following package-globals:
 
 =item C<$PS1> -- the default prompt
 
-=item C<$STOPDIE> -- true to enter the inspector on C<die()>
-
-=item C<$STOPWARN> -- true to enter the inspector on C<warn()>
-
 =item C<$STRICT> -- whether 'use strict' is applied to input
 
 =item C<$WANTARRAY> -- evaluation context
 
-=item C<$PRINT_PRETTY> -- format some output nicely (default = 0)
+=item C<$PRINT_PRETTY> -- format some output nicely (default = 1)
 
 Format some values nicely, independent of $PRINTER.  Currently, this
 displays arrays of scalars as columns.
@@ -591,13 +652,10 @@ displays arrays of scalars as columns.
 BEGIN {
     no strict;
     $PS1 = "> ";
-    $dies = 0;
-    $STOPDIE = 1;
-    $STOPWARN = 0;
     $PACKAGE = 'main';
     $WANTARRAY = 1;
     $PRINTER = \&Sepia::print_dumper;
-    $PRINT_PRETTY = 0;
+    $PRINT_PRETTY = 1;
     %REPL = (help => \&Sepia::repl_help,
              cd => \&Sepia::repl_chdir,
              methods => \&Sepia::repl_methods,
@@ -607,29 +665,38 @@ BEGIN {
              format => \&Sepia::repl_format,
              strict => \&Sepia::repl_strict,
              quit => \&Sepia::repl_quit,
+             reload => \&Sepia::repl_reload,
+             shell => \&Sepia::repl_shell,
          );
     %REPL_DOC = (
         cd =>
-    'cd DIR            Change directory to DIR',
+    'cd DIR             Change directory to DIR',
         format =>
     'format [dumper|dump|yaml|plain]
                        Set output formatter (default: dumper)',
         help =>
     'help               Display this message',
-        methods =>
-    'methods X          List methods for reference or package X',
+        methods => <<EOS,
+methods X [RE]     List methods for reference or package X,
+                       matching optional pattern RE.
+EOS
         package =>
     'package PACKAGE    Set evaluation package to PACKAGE',
         quit =>
     'quit               Quit the REPL',
+        shell =>
+    'shell CMD ...      Run CMD in the shell.',
         strict =>
     'strict [0|1]       Turn \'use strict\' mode on or off',
         wantarray =>
     'wantarray [0|1]    Set or toggle evaluation context',
-        who =>
-    'who PACKAGE        List variables and subs in PACKAGE',
+        who => <<EOS,
+who PACKAGE [RE]   List variables and subs in PACKAGE matching optional
+                       pattern RE.
+EOS
+        reload =>
+    'reload             Reload Sepia.pm and relaunch the REPL.',
     );
-    %RK = abbrev keys %REPL;
 }
 
 sub prompt()
@@ -643,42 +710,6 @@ sub Dump {
     };
 }
 
-sub eval_in_env
-{
-    my ($expr, $env) = @_;
-    local $::ENV = $env;
-    my $str = '';
-    for (keys %$env) {
-        next unless /^([\$\@%])(.+)/;
-        $str .= "local *$2 = \$::ENV->{'$_'}; ";
-    }
-    eval "do { no strict; $str $expr }";
-}
-
-sub debug_upeval
-{
-    my ($lev, $exp) = $_[0] =~ /^\s*(\d+)\s+(.*)/;
-    print " <= $exp\n";
-    (0, eval_in_env($exp, peek_my(0+$lev)));
-}
-
-sub debug_inspect
-{
-    local $_ = shift;
-    for my $i (split) {
-        my $sub = (caller $i)[3];
-        next unless $sub;
-        my $h = peek_my($i);
-        print "[$i] $sub:\n";
-        no strict;
-        for (sort keys %$h) {
-            local @res = $h->{$_};
-            print "\t$_ = ", $PRINTER->(), "\n";
-        }
-    }
-    0;
-}
-
 sub repl_help
 {
     print "REPL commands (prefixed with ','):\n";
@@ -722,8 +753,8 @@ sub repl_chdir
 
 sub who
 {
-    my ($pack, $re) = (shift =~ /^(\S+)(?:\s+(\S.*))?/);
-    $re ||= '';
+    my ($pack, $re) = @_;
+    $re ||= '.?';
     $re = qr/$re/;
     no strict;
     sort grep /$re/, map {
@@ -757,28 +788,37 @@ sub columnate
 
 sub repl_who
 {
-    print columnate who @_;
+    my ($pkg, $re) = split ' ', shift;
+    print columnate who($pkg || $PACKAGE, $re);
     0;
 }
 
 sub methods
 {
-    my $pack = shift;
+    my ($pack, $qualified) = @_;
     no strict;
-    (grep(defined *{"$pack\::$_"}{CODE}, keys %{$pack.'::'}),
-     defined @{$pack.'::ISA'} ? (map methods($_), @{$pack.'::ISA'}) : ());
+    my @own = $qualified ? grep {
+        defined *{$_}{CODE}
+    } map { "$pack\::$_" } keys %{$pack.'::'}
+        : grep {
+            defined *{"$pack\::$_"}{CODE}
+        } keys %{$pack.'::'};
+    (@own, defined @{$pack.'::ISA'}
+         ? (map methods($_, $qualified), @{$pack.'::ISA'}) : ());
 }
 
 sub repl_methods
 {
-    my $x = shift;
+    my ($x, $re) = split ' ', shift;
     $x =~ s/^\s+//;
     $x =~ s/\s+$//;
     if ($x =~ /^\$/) {
         $x = repl_eval("ref $x");
         return 0 if $@;
     }
-    print columnate sort { $a cmp $b } methods $x;
+    $re ||= '.?';
+    $re = qr/$re/;
+    print columnate sort { $a cmp $b } grep /$re/, methods $x;
     0;
 }
 
@@ -814,28 +854,22 @@ sub repl_quit
     1;
 }
 
-sub debug_help
+sub repl_reload
 {
-    print <<EOS;
-Inspector commands (prefixed with ','):
-    \\C-c        Pop one debugger level
-    backtrace       show backtrace
-    inspect N ...   inspect lexicals in frame(s) N ...
-    eval N EXPR     evaluate EXPR in lexical environment of frame N
-    return EXPR     return EXPR
-    die/warn        keep on dying/warning
-EOS
-    0;
+    do $INC{'Sepia.pm'};
+    if ($@) {
+        print "Reload failed:\n$@\n";
+    } else {
+        @_ = (select, 0);
+        goto &Sepia::repl;
+    }
 }
 
-sub debug_backtrace
+sub repl_shell
 {
-    Carp::cluck;0
-}
-
-sub debug_return
-{
-    (1, repl_eval(@_));
+    my $cmd = shift;
+    print `$cmd 2>& 1`;
+    return 0;
 }
 
 sub repl_eval
@@ -865,10 +899,40 @@ sub repl_eval
     }
 }
 
+## Collects warnings for REPL
+my @warn;
+
+sub sig_warn
+{
+    push @warn, shift
+}
+
+sub print_warnings
+{
+    if (@warn) {
+        if ($ISEVAL) {
+            my $tmp = "@warn";
+            print ';;;'.length($tmp)."\n$tmp\n";
+        } else {
+            for (@warn) {
+                # s/(.*) at .*/$1/;
+                print "warning: $_\n";
+            }
+        }
+    }
+}
+
 sub repl
 {
-    my ($fh, $level) = @_;
-    select((select($fh), $|=1)[0]);
+    if (@_ > 0) {
+        $REPL_IN = $_[0];
+        $REPL_OUT = $_[1];
+    }
+    select $REPL_OUT;
+    $| = 1;
+
+    local $REPL_LEVEL = $REPL_LEVEL + 1;
+
     my $in;
     my $buf = '';
     my $sigged = 0;
@@ -876,43 +940,10 @@ sub repl
     my $nextrepl = sub { $sigged = 1; };
 
     local *__;
-    my $MSG = "('\\C-c' to exit, ',h' for help)";
-    my %dhooks = (
-                backtrace => \&Sepia::debug_backtrace,
-                inspect => \&Sepia::debug_inspect,
-                eval => \&Sepia::debug_upeval,
-                return => \&Sepia::debug_return,
-                help => \&Sepia::debug_help,
-            );
-    local *CORE::GLOBAL::die = sub {
-        my @dieargs = @_;
-        if ($STOPDIE) {
-            local $dies = $dies+1;
-            local $PS1 = "*$dies*> ";
-            no strict;
-            local %Sepia::REPL = (
-                %dhooks, die => sub { local $Sepia::STOPDIE=0; die @dieargs });
-            local %Sepia::RK = abbrev keys %Sepia::REPL;
-            print "@_\nDied $MSG\n";
-            return Sepia::repl($fh, 1);
-        }
-        CORE::die(@_);
-    };
-
-    local *CORE::GLOBAL::warn = sub {
-        if ($STOPWARN) {
-            local $dies = $dies+1;
-            local $PS1 = "*$dies*> ";
-            no strict;
-            local %Sepia::REPL = (
-                %dhooks, warn => sub { local $Sepia::STOPWARN=0; warn @dieargs });
-            local %Sepia::RK = abbrev keys %Sepia::REPL;
-            print "@_\nWarned $MSG\n";
-            return Sepia::repl($fh, 1);
-        }
-        CORE::warn(@_);
-    };
-    print <<EOS;
+    local *CORE::GLOBAL::die = \&Sepia::Debug::die;
+    local *CORE::GLOBAL::warn = \&Sepia::Debug::warn;
+    Sepia::Debug::add_repl_commands;
+    print <<EOS if $REPL_LEVEL == 1;
 Sepia version $Sepia::VERSION.
 Press ",h" for help, or "^D" or ",q" to exit.
 EOS
@@ -920,7 +951,7 @@ EOS
     my @sigs = qw(INT TERM PIPE ALRM);
     local @SIG{@sigs};
     $SIG{$_} = $nextrepl for @sigs;
- repl: while (my $in = <$fh>) {
+ repl: while (defined(my $in = <$REPL_IN>)) {
             if ($sigged) {
                 $buf = '';
                 $sigged = 0;
@@ -928,20 +959,24 @@ EOS
                 next repl;
             }
             $buf .= $in;
-            my $iseval;
+            $buf =~ s/^\s*//;
+            local $ISEVAL;
             if ($buf =~ /^<<(\d+)\n(.*)/) {
-                $iseval = 1;
+                $ISEVAL = 1;
                 my $len = $1;
                 my $tmp;
                 $buf = $2;
-                while ($len && defined($tmp = read $fh, $buf, $len, length $buf)) {
+                while ($len && defined($tmp = read $REPL_IN, $buf, $len, length $buf)) {
                     $len -= $tmp;
                 }
             }
-            my (@res, @warn);
-            local $SIG{__WARN__} = sub {
-                push @warn, shift;
-            };
+            my (@res);
+            ## Only install a magic handler if no one else is playing.
+            local $SIG{__WARN__} = $SIG{__WARN__};
+            @warn = ();
+            unless ($SIG{__WARN__}) {
+                $SIG{__WARN__} = 'Sepia::sig_warn';
+            }
             if ($buf =~ /^,(\S+)\s*(.*)/s) {
                 ## Inspector shortcuts
                 my $short = $1;
@@ -968,9 +1003,15 @@ EOS
             } else {
                 ## Ordinary eval
                 @res = repl_eval $buf, wantarray;
-
                 if ($@) {
-                    if ($@ =~ /at EOF$/m) {
+                    if ($ISEVAL) {
+                        ## Always return results for an eval request
+                        Sepia::printer \@res, wantarray;
+                        Sepia::printer [$@], wantarray;
+                        # print_warnings $ISEVAL;
+                        $buf = '';
+                        print prompt;
+                    } elsif ($@ =~ /at EOF$/m) {
                         ## Possibly-incomplete line
                         if ($in eq "\n") {
                             print "Error:\n$@\n*** cancel ***\n", prompt;
@@ -978,28 +1019,23 @@ EOS
                         } else {
                             print ">> ";
                         }
-                        next repl;
                     } else {
-                        warn $@;
+                        print_warnings;
+                        # $@ =~ s/(.*) at eval .*/$1/;
+                        print "error: $@\n";
+                        print prompt;
                         $buf = '';
-                        Sepia::printer \@res, $iseval, wantarray if $iseval;
                     }
+                    next repl;
                 }
             }
             if ($buf !~ /;$/ && $buf !~ /^,/) {
                 ## Be quiet if it ends with a semicolon, or if we
                 ## executed a shortcut.
-                Sepia::printer \@res, $iseval, wantarray;
+                Sepia::printer \@res, wantarray;
             }
             $buf = '';
-            if (@warn) {
-                if ($iseval) {
-                    my $tmp = "@warn";
-                    print ';;;'.length($tmp)."\n$tmp\n";
-                } else {
-                    print "@warn\n";
-                }
-            }
+            print_warnings;
             print prompt;
         }
 }
@@ -1029,31 +1065,31 @@ sub html_module_list
     my $inst = inst();
     return unless $inst;
     return unless open OUT, ">$file";
-    print "<html><body><ul>";
+    print OUT "<html><body><ul>";
     my $pfx = '';
     my %ns;
     for (package_list) {
         push @{$ns{$1}}, $_ if /^([^:]+)/;
     }
     for (sort keys %ns) {
-        print qq{<li><b>$_</b><ul>} if @{$ns{$_}} > 1;
+        print OUT qq{<li><b>$_</b><ul>} if @{$ns{$_}} > 1;
         for (sort @{$ns{$_}}) {
             my @fs = map {
                 s/.*man.\///; s|/|::|g; s/\..?pm//; $_
             } grep /\.\dpm$/, sort $inst->files($_);
             if (@fs == 1) {
-                print qq{<li><a href="$base$fs[0]">$fs[0]</a>};
+                print OUT qq{<li><a href="$base$fs[0]">$fs[0]</a>};
             } else {
-                print qq{<li>$_<ul>};
+                print OUT qq{<li>$_<ul>};
                 for (@fs) {
-                    print qq{<li><a href="$base$_">$_</a>};
+                    print OUT qq{<li><a href="$base$_">$_</a>};
                 }
-                print '</ul>';
+                print OUT '</ul>';
             }
         }
-        print qq{</ul>} if @{$ns{$_}} > 1;
+        print OUT qq{</ul>} if @{$ns{$_}} > 1;
     }
-    print "</ul></body></html>\n";
+    print OUT "</ul></body></html>\n";
     close OUT;
     1;
 }
