@@ -29,8 +29,10 @@ most of its code.
 
 =cut
 
+# use Sepia '_apropos_re';
+require Sepia;
 BEGIN { *_apropos_re = *Sepia::_apropos_re; }
-$VERSION = '0.64';
+$VERSION = '0.65';
 
 use strict;
 use Config;
@@ -38,8 +40,6 @@ use Cwd 'abs_path';
 use B qw(peekop class comppadlist main_start svref_2object walksymtable
          OPpLVAL_INTRO SVf_POK OPpOUR_INTRO OPf_MOD OPpDEREF_HV OPpDEREF_AV
 	 cstring);
-# use Sepia '_apropos_re';
-require Sepia;
 
 =head2 Variables
 
@@ -429,7 +429,7 @@ sub pp_method_named {
 	$top = [$lastclass || "(method)", '->', $name];
 	undef $lastclass;
     } else {
-	warn "method_named: wtf: sizeof padval = ".@padval;
+	dprint 'method_named', "method_named: wtf: sizeof padval = ".@padval;
     }
 }
 
@@ -605,7 +605,8 @@ sub _var_ret_list
     if ($mod) {
         @r = exists $h->{$mod} ? @{$h->{$mod}} : ();
     } else {
-       @r = map { @$_ } values %$h;
+        ## XXX: Need to revisit when this is/isn't an array!
+        @r = map { ref $_ eq 'ARRAY' ? @$_ : $_ } values %$h;
     }
     @r = grep $_->{assign}, @r if $assign;
     @r = map { [@{$_}{qw(file line sub package)}] } @r;
