@@ -18,7 +18,7 @@
 ;;; Code:
 
 (require 'cperl-mode)
-(require 'comint)
+(require 'gud)
 (require 'cl)
 ;; try optional modules, but don't bitch if we fail:
 (require 'sepia-w3m nil t)
@@ -29,7 +29,7 @@
 ;;; Comint communication
 
 (defvar sepia-perl5lib nil
-"* Extra PERL5LIB directory for Sepia.pm")
+"* List of extra PERL5LIB directories for `sepia-repl'.")
 
 (defvar sepia-program-name "perl"
 "* Perl program name.")
@@ -42,7 +42,7 @@ Useful values include `w3m-perldoc' and `cperl-perldoc'.")
 
 (defvar sepia-view-pod-function
   (if (featurep 'w3m) 'sepia-w3m-view-pod 'sepia-perldoc-buffer)
-"* Function to view modules' documentation.
+"* Function to view current buffer's documentation.
 
 Useful values include `sepia-w3m-view-pod' and `sepia-perldoc-buffer'.")
 
@@ -63,17 +63,19 @@ attempted for objects that are simple scalars.")
 "* If non-NIL, `sepia-indent-or-complete' tries `expand-abbrev'.")
 
 (defvar sepia-use-completion t
-  "* Use completion based on Xref database.  Turning this off may
-speed up some operations, if you don't mind losing completion.")
+  "* Use completion based on Xref database.
+
+Turning this off may speed up some operations, if you don't mind
+losing completion.")
 
 (defvar sepia-eval-defun-include-decls t
-  "* Generate and use a declaration list for ``sepia-eval-defun''.
+  "* Generate and use a declaration list for `sepia-eval-defun'.
 Without this, code often will not parse; with it, evaluation may
 be a bit less responsive.  Note that since this only includes
 subs from the evaluation package, it may not always work.")
 
 (defvar sepia-prefix-key "\M-."
-  "* Prefix for functions in ``sepia-keymap''.")
+  "* Prefix for functions in `sepia-keymap'.")
 
 ;;; User options end here.
 
@@ -198,26 +200,9 @@ each inferior Perl prompt."
     (define-key map "\C-c\C-d" 'sepia-view-pod)
     (define-key map "\C-c\C-r" 'sepia-repl)
     (define-key map "\C-c\C-s" 'sepia-scratch)
+    (define-key map "\C-c!" 'sepia-set-cwd)
     (define-key map (kbd "TAB") 'sepia-indent-or-complete)))
 
-(defun sepia-comint-setup ()
-"Set up the inferior Perl process buffer."
-  (comint-mode)
-  (set (make-local-variable 'comint-dynamic-complete-functions)
-       '(sepia-complete-symbol comint-dynamic-complete-filename))
-  (set (make-local-variable 'comint-preoutput-filter-functions)
-       '(sepia-watch-for-eval))
-  (set (make-local-variable 'comint-use-prompt-regexp) t)
-  (modify-syntax-entry ?: "_")
-  (modify-syntax-entry ?> ".")
-  (use-local-map (copy-keymap (current-local-map)))
-  (sepia-install-keys)
-  (local-set-key (kbd "TAB") 'comint-dynamic-complete)
-  (local-set-key "\C-a" 'comint-bol)
-  (set (make-local-variable 'comint-prompt-regexp)
-       "^[^>\n]*> *")
-  )
-
 ;;;###autoload
 (defun sepia-perldoc-this (name)
   "View perldoc for module at point."
@@ -225,15 +210,15 @@ each inferior Perl prompt."
   (let ((wc (current-window-configuration))
         (old-pd (symbol-function 'w3m-about-perldoc))
         (old-pdb (symbol-function 'w3m-about-perldoc-buffer)))
-    (flet ((w3m-about-perldoc (&rest args)
-             (let ((res (apply old-pd args)))
-               (or res (error "lose: %s" args))))
-           (w3m-about-perldoc-buffer (&rest args)
-             (let ((res (apply old-pdb args)))
-               (or res (error "lose: %s" args)))))
-      (condition-case stuff
-          (funcall sepia-perldoc-function name)
-        (error (set-window-configuration wc))))))
+    (condition-case stuff
+        (flet ((w3m-about-perldoc (&rest args)
+                 (let ((res (apply old-pd args)))
+                   (or res (error "lose: %s" args))))
+               (w3m-about-perldoc-buffer (&rest args)
+                 (let ((res (apply old-pdb args)))
+                   (or res (error "lose: %s" args)))))
+          (funcall sepia-perldoc-function name))
+      (error (set-window-configuration wc)))))
 
 (defun sepia-view-pod ()
   "View POD for the current buffer."
@@ -297,13 +282,70 @@ For modules within packages, see `sepia-module-list'."
                         "perl" sepia-program-name nil
                         (append (mapcar (lambda (x) (concat "-I" x))
                                         sepia-perl5lib)
-                                '("-MData::Dumper" "-MSepia" "-MSepia::Xref"
-                                  "-e" "Sepia::repl(*STDIN)")))))
-      (with-current-buffer "*sepia-repl*"
-        (sepia-comint-setup))
-      (accept-process-output sepia-process 0 1))
+                                '("-MSepia" "-MSepia::Xref"
+                                  "-e" "Sepia::repl(*STDIN, *STDOUT)")))))
+    (with-current-buffer "*sepia-repl*"
+      (sepia-repl-mode))
+    (accept-process-output sepia-process 0 1)
+    ;; Steal a bit from gud-common-init:
+    (setq gud-running t)
+    (setq gud-last-last-frame nil)
+    (set-process-filter sepia-process 'gud-filter)
+    (set-process-sentinel sepia-process 'gud-sentinel)
+    )
   (pop-to-buffer (get-buffer "*sepia-repl*")))
 
+(define-derived-mode sepia-repl-mode gud-mode "Sepia REPL"
+  "Major mode for the Sepia REPL."
+    (set (make-local-variable 'comint-dynamic-complete-functions)
+         '(sepia-complete-symbol comint-dynamic-complete-filename))
+    (set (make-local-variable 'comint-preoutput-filter-functions)
+         '(sepia-watch-for-eval))
+    ;; (set (make-local-variable 'comint-use-prompt-regexp) t)
+    (modify-syntax-entry ?: "_")
+    (modify-syntax-entry ?> ".")
+    ;; (use-local-map (copy-keymap (current-local-map)))
+    (sepia-install-keys sepia-repl-mode-map)
+    (define-key sepia-repl-mode-map
+        (kbd "<tab>") 'comint-dynamic-complete)
+    (define-key sepia-repl-mode-map "\C-a" 'comint-bol)
+    (set (make-local-variable 'comint-prompt-regexp) "^[^>\n]*> *")
+    (set (make-local-variable 'gud-target-name) "sepia")
+    (set (make-local-variable 'gud-marker-filter) 'sepia-gud-marker-filter)
+    (set (make-local-variable 'gud-minor-mode) 'sepia)
+
+    (setq gud-comint-buffer (current-buffer))
+    (setq gud-last-last-frame nil)
+    (setq gud-sepia-acc nil)
+
+    (gud-def gud-break ",break %f:%l" "\C-b" "Set breakpoint at current line.")
+    (gud-def gud-step ",step %p" "\C-s" "Step one line.")
+    (gud-def gud-next ",next %p" "\C-n" "Step one line, skipping calls.")
+    (gud-def gud-cont ",continue" "\C-r" "Continue.")
+    (gud-def gud-print "%e" "\C-p" "Evaluate something.")
+    (gud-def gud-remove ",delete %l %f" "\C-d" "Delete current breakpoint.")
+    (run-hooks 'sepia-repl-mode-hook))
+
+(defvar gud-sepia-acc nil
+  "Accumulator for `sepia-gud-marker-filter'.")
+
+(defun sepia-gud-marker-filter (str)
+  (setq gud-sepia-acc
+        (if gud-sepia-acc
+            (concat gud-sepia-acc str)
+            str))
+  (while (string-match "_<\\([^:>]+\\):\\([0-9]+\\)>\\(.*\\)" gud-sepia-acc)
+    (setq gud-last-last-frame gud-last-frame
+          gud-last-frame (cons
+                          (match-string 1 gud-sepia-acc)
+                          (string-to-number (match-string 2 gud-sepia-acc)))
+          gud-sepia-acc (match-string 3 gud-sepia-acc)))
+  (setq gud-sepia-acc
+        (if (string-match "\\(_<.*\\)" gud-sepia-acc)
+            (match-string 1 gud-sepia-acc)
+            nil))
+  str)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Xref
 
@@ -333,11 +375,11 @@ module in question be loaded.")))
                          mod ,pl-func))))))
 
 (defun sepia-thing-at-point (what)
-  "Like ``thing-at-point'', but hacked to avoid REPL prompt."
+  "Like `thing-at-point', but hacked to avoid REPL prompt."
   (let ((th (thing-at-point what)))
     (and th (not (string-match "[ >]$" th)) th)))
 
-(defvar sepia-sub-re "^\\s *sub\\s +\\(.+\\_>\\)")
+(defvar sepia-sub-re "^ *sub\\s +\\(.+\\_>\\)")
 
 (defvar sepia-history nil)
 
@@ -390,9 +432,11 @@ symbol at point."
 "Find the file defining module MOD."
   (interactive (list (sepia-interactive-arg 'module)))
   (let ((fn (sepia-find-module-file mod)))
-    (when fn
-      (message "Module %s in %s." mod fn)
-      (pop-to-buffer (find-file-noselect (expand-file-name fn))))))
+    (if fn
+        (progn
+          (message "Module %s in %s." mod fn)
+          (pop-to-buffer (find-file-noselect (expand-file-name fn))))
+        (message "Can't find module %s." mod))))
 
 (defmacro ifa (test then &rest else)
   `(let ((it ,test))
@@ -435,9 +479,9 @@ symbol at point."
   `(defun ,name (ident &optional module file line display-p)
      ,(concat doc "
 
-With prefix arg, list occurences in a ``grep-mode'' buffer.
-Without, place the occurrences on ``sepia-found'', so that
-calling ``sepia-next'' will cycle through them.
+With prefix arg, list occurences in a `grep-mode' buffer.
+Without, place the occurrences on `sepia-found', so that
+calling `sepia-next' will cycle through them.
 
 Depending on the query, MODULE, FILE, and LINE may be used to
 narrow the results, as long as doing so leaves some matches.
@@ -595,32 +639,47 @@ to this location."
 (defun sepia-beginning-of-defun (&optional n)
 "Move to beginning of current function.
 
-If prefix argument given, move N functions backward."
+The prefix argument is the same as for `beginning-of-defun'."
   (interactive "p")
-  (let ((here (point)))
-    (beginning-of-line)
-    (if (and (not (= here (point)))
-             (looking-at sepia-sub-re))
-        (point)
-        (sepia-safe-bodf n)
-        (let* ((end (point))
-               (beg (progn (forward-line -3) (point))))
-          (goto-char end)
-          (re-search-backward sepia-sub-re beg t)))))
+  (setq n (or n 1))
+  (ignore-errors
+    (when (< n 0)
+      (sepia-end-of-defun (- n))
+      (setq n 1))
+    (re-search-backward sepia-sub-re nil nil n)))
+
+(defun sepia-inside-defun ()
+  "True if point is inside a sub."
+  (condition-case nil
+      (save-excursion
+        (let ((cur (point)))
+          (re-search-backward sepia-sub-re)
+          (when (< (point) cur)
+            (search-forward "{")
+            (backward-char 1)
+            (forward-sexp)
+            (> (point) cur))))
+    (error nil)))
 
 (defun sepia-end-of-defun (&optional n)
   "Move to end of current function.
 
-If prefix argument given, move N functions forward."
+The prefix argument is the same as for `end-of-defun'."
   (interactive "p")
-  (let ((here (point)))
-    ;; (sepia-safe-bodf)
-    (when (looking-at sepia-sub-re)
-      (forward-line 1))
-    (sepia-safe-eodf n)
-    (when (and (>= here (point))
-               (re-search-forward sepia-sub-re nil t))
-      (sepia-safe-eodf))
+  (setq n (or n 1))
+  (when (< n 0)
+    (sepia-beginning-of-defun (- n))
+    (setq n 1))
+  ;; If we're outside a defun, skip to the next
+  (ignore-errors
+    (unless (sepia-inside-defun)
+      (re-search-forward sepia-sub-re)
+      (forward-char 1))
+    (dotimes (i n)
+      (re-search-backward sepia-sub-re)
+      (search-forward "{")
+      (backward-char 1)
+      (forward-sexp))
     (point)))
 
 (defun sepia-defun-around-point (&optional where)
@@ -744,13 +803,10 @@ also rebuild the xref database."
 (defun sepia-ident-before-point ()
   "Find the Perl identifier at or preceding point."
   (save-excursion
-    (when (looking-at "[%$@*&]")
-      (forward-char 1))
     (let* ((end (point))
            (beg (progn
-                 (when (re-search-backward "[^A-Za-z_0-9:]" nil 'mu)
-                   (forward-char 1))
-                 (point)))
+                  (skip-chars-backward "a-zA-Z0-9_:")
+                  (point)))
            (sigil (if (= beg (point-min))
                       nil
                       (char-before (point)))))
@@ -836,7 +892,7 @@ function currently ignores module qualifiers, which may be
 annoying in larger programs.
 
 The function is intended to be bound to \\M-TAB, like
-``lisp-complete-symbol''."
+`lisp-complete-symbol'."
   (interactive)
   (let ((win (get-buffer-window "*Completions*" 0))
         len
@@ -881,8 +937,8 @@ The function is intended to be bound to \\M-TAB, like
                                  (?& "CODE")
                                  (?* "IO")
                                  (t ""))
-                               (unless (eq major-mode 'comint-mode)
-                                 (sepia-function-at-point)))))
+                               (and (eq major-mode 'sepia-mode)
+                                    (sepia-function-at-point)))))
           ;; 3 - try a Perl built-in
           (when (and (not completions)
                      (or (not type) (eq type ?&)))
@@ -892,8 +948,6 @@ The function is intended to be bound to \\M-TAB, like
           (case (length completions)
             (0 (message "No completions for %s." name) nil)
             (1 ;; XXX - skip sigil to match s-i-before-point
-             (when (looking-at "[%$@*&]")
-               (forward-char 1))
              (delete-region (- (point) len) (point))
              (insert (or type "") (car completions))
              ;; Hide stale completions buffer (stolen from lisp.el).
@@ -930,7 +984,7 @@ This function is intended to be bound to TAB."
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; scratchpad code
 
-(defvar sepia-mode-map nil "Keymap for Sepia mode.")
+;; (defvar sepia-mode-map nil "Keymap for Sepia mode.")
 
 (defvar sepia-metapoint-map nil
   "Keymap for Sepia functions.  This is just an example of how you
@@ -947,8 +1001,7 @@ might want to bind your keys, which works best when bound to
   (set (make-local-variable 'beginning-of-defun-function)
        'sepia-beginning-of-defun)
   (set (make-local-variable 'end-of-defun-function)
-       'sepia-end-of-defun)
-  (sepia-init))
+       'sepia-end-of-defun))
 
 (defun sepia-init ()
   "Perform the initialization necessary to start Sepia."
@@ -1160,18 +1213,22 @@ With prefix arg, replace the region with the result."
 (defun sepia-eval-defun ()
   "Re-evaluate the current function and rebuild its Xrefs."
   (interactive)
-  (let (pt end beg sub res)
+  (let (pt end beg sub res
+           sepia-eval-package
+           sepia-eval-file
+           sepia-eval-line)
     (save-excursion
       (setq pt (point)
             end (progn (end-of-defun) (point))
-            beg (progn (goto-char pt) (beginning-of-defun) (point)))
+            beg (progn (beginning-of-defun) (point)))
       (goto-char beg)
       (when (looking-at "^sub\\s +\\(.+\\_>\\)")
-        (let ((sepia-eval-package (sepia-guess-package sub (buffer-file-name)))
-              (body (buffer-substring-no-properties beg end))
-              (sepia-eval-file (buffer-file-name))
-              (sepia-eval-line (line-number-at-pos beg)))
-          (setq sub (match-string 1)
+        (setq sub (match-string 1))
+        (let ((body (buffer-substring-no-properties beg end)))
+          
+          (setq sepia-eval-package (sepia-guess-package sub (buffer-file-name))
+                sepia-eval-file (buffer-file-name)
+                sepia-eval-line (line-number-at-pos beg)
                 res
                 (sepia-eval-raw
                  (if sepia-eval-defun-include-decls
@@ -1221,11 +1278,15 @@ With prefix arg, replace the region with the result."
 ;; REPL
 
 (defvar sepia-eval-file nil
-  "File in which ``sepia-eval'' evaluates perl expressions.")
+  "File in which `sepia-eval' evaluates perl expressions.")
 (defvar sepia-eval-line nil
-  "Line at which ``sepia-eval'' evaluates perl expressions.")
+  "Line at which `sepia-eval' evaluates perl expressions.")
 
 (defun sepia-set-cwd (dir)
+  "Set the inferior Perl process's working directory to DIR.
+
+When called interactively, the current buffer's
+`default-directory' is used."
   (interactive (list default-directory))
   (sepia-call "Cwd::chdir" dir))
 
@@ -1239,37 +1300,48 @@ With prefix arg, replace the region with the result."
 (defun sepia-doc-scan-buffer ()
   (save-excursion
     (goto-char (point-min))
-    (loop while (re-search-forward
+    (loop
+       while (re-search-forward
 		 "^=\\(item\\|head[2-9]\\)\\s +\\([%$&@A-Za-z_].*\\)" nil t)
-       if (ignore-errors
-            (let* ((s1 (match-string 2))
-                   (s2 (let ((case-fold-search nil))
-                         (replace-regexp-in-string
-                          "[A-Z]<\\([^>]+\\)>" "\\1" s1)))
-                   (longdoc
+       if
+         (ignore-errors
+           (let ((short (match-string 2)) longdoc)
+              (setq short
+                    (let ((case-fold-search nil))
+                      (replace-regexp-in-string
+                       "E<lt>" "<"
+                       (replace-regexp-in-string
+                        "E<gt>" ">"
+                        (replace-regexp-in-string
+                         "[A-DF-Z]<\\([^<>]+\\)>" "\\1" short)))))
+              (while (string-match "^\\s *[A-Z]<\\(.*\\)>\\s *$" short)
+                (setq short (match-string 1 short)))
+              (setq longdoc
                     (let ((beg (progn (forward-line 2) (point)))
                           (end (1- (re-search-forward "^=" nil t))))
                       (forward-line -1)
                       (goto-char beg)
                       (if (re-search-forward "^\\(.+\\)$" end t)
-                          (concat s2 ": "
+                          (concat short ": "
                                   (substring-no-properties
                                    (match-string 1)
                                    0 (position ?. (match-string 1))))
-                          s2))))
+                          short)))
               (cond
                 ;; e.g. "$x -- this is x"
                 ((string-match "^[%$@]\\([A-Za-z0-9_:]+\\)\\s *--\\s *\\(.*\\)"
-                               s2)
-                 (list 'variable (match-string-no-properties 1 s2)
-                       (or (and (equal s2 (match-string 1 s2)) longdoc) s2)))
+                               short)
+                 (list 'variable (match-string-no-properties 1 short)
+                       (or (and (equal short (match-string 1 short)) longdoc)
+                           short)))
                 ;; e.g. "C<foo(BLAH)>" or "$x = $y->foo()"
-                ((string-match "\\([A-Za-z0-9_:]+\\)\\s *\\(\\$\\|(\\)" s2)
-                 (list 'function (match-string-no-properties 1 s2)
-                       (or (and (equal s2 (match-string 1 s2)) longdoc) s2)))
+                ((string-match "\\([A-Za-z0-9_:]+\\)\\s *\\(\\$\\|(\\)" short)
+                 (list 'function (match-string-no-properties 1 short)
+                       (or (and (equal short (match-string 1 short)) longdoc)
+                           short)))
                 ;; e.g. "$x this is x" (note: this has to come last)
-                ((string-match "^[%$@]\\([^( ]+\\)" s2)
-                 (list 'variable (match-string-no-properties 1 s2) longdoc)))))
+                ((string-match "^[%$@]\\([^( ]+\\)" short)
+                 (list 'variable (match-string-no-properties 1 short) longdoc)))))
        collect it)))
 
 (defun sepia-buffer-package ()
@@ -1302,8 +1374,8 @@ used for eldoc feedback."
 (defun sepia-symbol-info (&optional obj type)
   "Eldoc function for Sepia-mode.
 
-Looks in ``sepia-doc-map'' and ``sepia-var-doc-map'', then tries
-calling ``cperl-describe-perl-symbol''."
+Looks in `sepia-doc-map' and `sepia-var-doc-map', then tries
+calling `cperl-describe-perl-symbol'."
   (unless obj
     (multiple-value-bind (ty ob) (sepia-ident-at-point)
       (setq obj (if (consp ob) (car ob) ob)
